Imports System.Net
Imports System.IO
Imports System.Text

Public Class RDNADownloader
    Inherits BaseDownloader


    Private Const BASE_PAGE_URL As String = "http://www.runtimedna.com/mod/bcs/index.php?ViewProduct={0}"
    Private Const IMAGE_URL As String = "http://www.runtimedna.com/mod/bcs/photos/{0}"

    Private Const URL_TO_REMOVE As String = "http://www.runtimedna.com/mod/bcs/index.php?ViewProduct="

    Private Const PRODUCT_CODE_START_MATCH As String = "<p><li><i>SKU:"
    Private Const PRODUCT_CODE_END_MATCH As String = "</i></li>"

    Private Const IMAGE_URL_START_MATCH As String = "</H3><IMG SRC=""photos/"
    Private Const IMAGE_URL_END_MATCH As String = """><BR><BR>"

    Private Const PRICE_START_MATCH As String = "<tr><td align=""right"">List Price:</td><td style=""font-weight:bold;"">$"
    Private Const PRICE_END_MATCH As String = "</td></tr>"

    Private Const STRIKE_START As String = "<strike>"
    Private Const STRIKE_END As String = "</strike>"

    Private Const PRODUCT_NAME_START_MATCH As String = "<H3>"
    Private Const PRODUCT_NAME_END_MATCH As String = "</H3>"

    Private Const ARTIST_START_MATCH As String = ""
    Private Const ARTIST_END_MATCH As String = ""

    Private Const REQUIREMENTS_START_MATCH As String = "<B>Requires: <A HREF="
    Private Const REQUIREMENTS_START_END_MATCH As String = """>"
    Private Const REQUIREMENTS_END_MATCH As String = "</A>"

    Public Const LINK_FILENAME As String = "RdnaLinks.xml"

    Public Overrides ReadOnly Property BasePageUrl() As String
        Get
            Return BASE_PAGE_URL
        End Get
    End Property

    Public Overrides ReadOnly Property BaseImageUrl() As String
        Get
            Return IMAGE_URL
        End Get
    End Property

    Public Sub New(ByVal imageDirectory As String, _
                   ByVal imageHolder As PictureBox)
        MyBase.New(imageDirectory, _
                    imageHolder)
    End Sub

    Public Overrides ReadOnly Property LinkFilename() As String
        Get
            Return LINK_FILENAME
        End Get
    End Property

    Public Overrides ReadOnly Property UrlToRemove() As String
        Get
            Return URL_TO_REMOVE
        End Get
    End Property

    Public Overrides Function DownloadImage(ByVal productCode As String, _
                                            ByVal productUrl As String) As String

        Dim productId As Integer = 0
        Try
            productId = CType(productUrl.Replace(URL_TO_REMOVE, EMPTY), Integer)
        Catch ex As Exception
            Throw New InvalidProductUrlException
        End Try

        Dim pageContents As String = GetPageContents(productId)
        Dim imageNameStart As Integer = pageContents.IndexOf(IMAGE_URL_START_MATCH)
        If imageNameStart > -1 Then
            pageContents = pageContents.Substring(imageNameStart)
            pageContents = pageContents.Substring(0, pageContents.IndexOf(IMAGE_URL_END_MATCH))
            pageContents = pageContents.Replace(IMAGE_URL_START_MATCH, EMPTY)
            pageContents = pageContents.Trim
            Try
                Dim fileName As String = My.Settings.ImageDirectory
                fileName &= productCode
                fileName &= JPEG_EXTENSION

                _imageHolder.Load(String.Format(BaseImageUrl, pageContents))
                _imageHolder.Image.Save(fileName, _
                                        System.Drawing.Imaging.ImageFormat.Jpeg)
                Application.DoEvents()
            Catch ex As System.Net.WebException
                If ex.Status <> WebExceptionStatus.ProtocolError Then
                    MsgBox(ex.Message)
                End If
            Catch ex As Exception
                MsgBox(ex.Message)
            Finally

            End Try
        End If

        Return productCode & JPEG_EXTENSION

    End Function


    Private Function GetProductName(ByVal pageContents As String) As String

        Dim productCodeStart As Integer = pageContents.IndexOf(PRODUCT_NAME_START_MATCH)
        Dim productName As String = ""

        If productCodeStart > -1 Then
            pageContents = pageContents.Substring(productCodeStart)
            pageContents = pageContents.Substring(0, pageContents.IndexOf(PRODUCT_NAME_END_MATCH))
            pageContents = pageContents.Replace(PRODUCT_NAME_START_MATCH, EMPTY)
            productName = pageContents.Trim
        End If

        Return productName

    End Function

    Private Function GetPrice(ByVal pageContents As String) As Single

        Dim productCodeStart As Integer = pageContents.IndexOf(PRICE_START_MATCH)
        Dim price As Single = 0.0

        If productCodeStart > -1 Then
            pageContents = pageContents.Substring(productCodeStart)
            pageContents = pageContents.Substring(0, pageContents.IndexOf(PRICE_END_MATCH))
            pageContents = pageContents.Replace(PRICE_START_MATCH, EMPTY)
            pageContents = pageContents.Replace(STRIKE_START, EMPTY)
            pageContents = pageContents.Replace(STRIKE_END, EMPTY)
            Try
                price = CType(pageContents, Single)
            Catch ex As Exception
                MsgBox(ex.Message)
            End Try

        End If

        Return price

    End Function

    Private Function GetRequiredProducts(ByVal pageContents As String) As String

        Dim requiredProducts As String = ""
        Dim requiredProductTemp As String = ""

        Dim requiredProductsStart As Integer = pageContents.IndexOf(REQUIREMENTS_START_MATCH)

        'If requiredProductsStart > -1 Then
        '    pageContents = pageContents.Substring(requiredProductsStart)
        '    pageContents = pageContents.Substring(0, pageContents.IndexOf(REQUIRED_PRODUCT_END_MATCH))
        '    pageContents = pageContents.Replace(REQUIRED_PRODUCT_START_MATCH, EMPTY)
        '    requiredProductsStart = pageContents.IndexOf(REQUIRED_PRODUCT_SEPERATOR_START_END)
        '    While requiredProductsStart > -1
        '        requiredProductTemp = pageContents.Substring(requiredProductsStart)
        '        requiredProductTemp = requiredProductTemp.Substring(0, requiredProductTemp.IndexOf(REQUIRED_PRODUCT_SEPERATOR_END))
        '        requiredProductTemp = requiredProductTemp.Replace(REQUIRED_PRODUCT_SEPERATOR_START_END, EMPTY)
        '        If requiredProducts.Length > 0 Then
        '            requiredProducts &= REQUIRED_PRODUCT_SEPERATOR
        '        End If
        '        requiredProducts &= requiredProductTemp
        '        pageContents = pageContents.Substring(pageContents.IndexOf(REQUIRED_PRODUCT_SEPERATOR_END) + REQUIRED_PRODUCT_SEPERATOR_END.Length)
        '        requiredProductsStart = pageContents.IndexOf(REQUIRED_PRODUCT_SEPERATOR_START_END)
        '    End While
        'End If

        Return requiredProducts

    End Function

    Protected Overrides Sub DownLoadPage(ByVal itemId As Integer)

        Try

            Dim pageContents As String = GetPageContents(itemId)
            Dim requiredProducts As String = ""
            Dim productName As String = ""
            Dim price As Single = 0.0

            Dim productCodeStart As Integer = pageContents.IndexOf(PRODUCT_CODE_START_MATCH)

            If productCodeStart > -1 Then

                requiredProducts = GetRequiredProducts(pageContents)
                price = GetPrice(pageContents)
                productName = GetProductName(pageContents)

                pageContents = pageContents.Substring(productCodeStart)
                pageContents = pageContents.Substring(0, pageContents.IndexOf(PRODUCT_CODE_END_MATCH))
                pageContents = pageContents.Replace(PRODUCT_CODE_START_MATCH, EMPTY)
                pageContents = pageContents.Trim

                _linkFile.AddLink(pageContents, _
                              String.Format(BASE_PAGE_URL, itemId.ToString), _
                              itemId, _
                              requiredProducts, _
                              productName, _
                              price)

            End If

        Catch ex As System.Net.WebException
            Throw
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Sub


    Public Overrides Sub Download()

        Try
            Dim MinRdnaId As Integer = _linkFile.MaxId + 1

            Download(MinRDNAId, _
                     My.Settings.MaxRDNAItemId)
        Catch ex As Exception

        End Try
        _linkFile.WriteFile(LINK_FILENAME)

    End Sub

End Class
